// TODO(ochafik): Move to dart.tools.pub.
library scissors.google3_util;

import 'dart:async';
import 'dart:io';

import 'path_resolver.dart';
import 'path_utils.dart';

// import 'package:dart.tools.pub/transformer_util.dart' as pubUtil;
var pubUtil;

const _google3PseudoPackage = 'google3';

class Google3PathResolver extends PathResolver {
  @override final String defaultSassCPath = 'third_party/sassc/sassc';

  @override final String defaultCssJanusPath =
      'third_party/wikidata/mediawiki/maintenance/cssjanus/cssjanus.par';

  @override final String defaultJRubyPath =
      'third_party/java/jruby/jruby_binary';

  @override final String defaultRubySassPath =
      'third_party/ruby/sass/sass_with_compass';

  @override final String defaultCompassStylesheetsPath =
      'third_party/ruby/compass/core/stylesheets';

  List<Directory> _rootDirectories;

  @override
  Future<List<Directory>> get rootDirectories async {
    if (_rootDirectories == null) {
      _rootDirectories = []
        ..addAll(await _google3RootDirectories)
        ..add(Directory.current);
    }
    return _rootDirectories;
  }
}

Future<List<Directory>> _google3RootDirectories = (() async {
  var dirs = <Directory>[];
  var runFiles = Platform.environment['RUNFILES'];
  if (runFiles != null) {
    // Note: we break out of pub_bin's RUNFILES to find the binaries.
    // We could find the binary at ${RUNFILES}/google3/third_party/sassc/sassc
    // if we added //third_party/sassc/sassc to the pub_bin target's data.
    // Also, $(location //third_party/sassc:sassc) is not working,
    // maybe because the current directory is changed by pub.
    //dirs.add('$runFiles');
    var baseDir = '$runFiles/../../../..';
    dirs.add(new Directory(baseDir));

    // Find all the runfiles directories.
    // TODO(ochafik): Find a clean way to get the dart_application's runfiles
    // directory when running in forge (the RUNFILES env var is currently set
    // to pub_bin.runfiles).
    var baseFiles =
        new Directory(baseDir).list(recursive: true, followLinks: false);
    await for (var f in baseFiles) {
      if (f.path.endsWith('.runfiles')) {
        dirs.add(new Directory('${f.path}/google3'));
      }
    }
  } else {
    var google3Dir = await pubUtil.getClientGoogle3Directory();
    dirs
      ..add(new Directory(google3Dir))
      ..add(new Directory('$google3Dir/blaze-bin'))
      ..add(new Directory('$google3Dir/blaze-genfiles'))
      ..add(new Directory('$google3Dir/blaze-out'))
      ..add(new Directory('$google3Dir/blaze-google3'));

    var git5Readonly = '$google3Dir/../READONLY/google3';
    if (await new Directory(git5Readonly).exists()) {
      dirs.add(new Directory(git5Readonly));
    }
  }

  var result = <String>[];
  await for (var dir in findExistingDirectories(dirs)) {
    result.add(dir);
  }

//   for (var dir in result) {
//     print("# dir: $dir");
//     await for (var f in new Directory(dir).list(recursive: true, followLinks: true)) {
//       if (f.path.contains("dashboard_common"))
//         print("#   $f");
//     }
//   }
  return result;
})();
